//------------------------------------------------------------------------------
// Bezier Splining 3 order for polygons
//  - XII/2001 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------

{
@abstract(Bezier Splining 3 order for polygons.)
@author  (Alexander Weidauer <alex.weidauer@huckfinn.de)
@created (December 2001)
@lastmod (December 2001)
The Unit  delivers  a smoothed bezier spline of a polygon in two
ways tangen modus and secant modus for data segments. }
Unit UBezierSpline;
//--------------------------------------------------------------------------
Interface
//--------------------------------------------------------------------------
Uses SysUtils,
  UConst,
  UXYPoint,
  UXYPointList;
//--------------------------------------------------------------------------

Type
{ Mode of your bezier operation. }
TBZOperationMode = (tbzSmooth, tbzExact);

{
 Bezier Polygon will work as follos:
 OpeartionMode  - tbzSmooth secant modus / tbzExact tangent modus;
 Closed - true assume the polygon is closed the first
          and the last point will be connected
 VC - the Polygon withc is used
 PointStep - Distance of each point   }
Procedure BezierPolygon(OperationMode: TBZOperationMode;
  Closed: TBoolean;
  VC: TXYPointList;
  PointStep: TDouble);
//--------------------------------------------------------------------------
Implementation

//--------------------------------------------------------------------------
{Determenig the bezier coefficientns.}
Function Bezier(x: TDouble; j: byte): TDouble;
Begin
  Bezier := 0;
  Case j Of 0: Begin x := 1 - x; Bezier := x * x * x / 6 End;
    1: Bezier := x * x * (x / 2 - 1) + 4 / 6;
    2: Bezier := x * (1 + x * (1 - x)) / 2 + 1 / 6;
    3: Bezier := x * x * x / 6;
    10: Bezier := -(1 - x) * (1 - x) * x / 2;
    11: Bezier := 1 + x * x * (-5 + 3 * x) / 2;
    12: Bezier := x * (1 + x * (4 - 3 * x)) / 2;
    13: Bezier := -(1 - x) * x * x / 2;
  End;
End;
//--------------------------------------------------------------------------
{Procedure to create the points}
Procedure BezierPolygon(OperationMode: TBZOperationMode;
  Closed: TBoolean;
  VC: TXYPointList;
  PointStep: TDouble);
Var i: TInt32;
  j: TInt32;
  k: TInt32; // operation mode variable
  l, step,
    sc, x, y: TDouble;
  Point: TXYPoint;
  VCTemp: TXYpointList;
Begin
  If VC.MaxPoint < 2 Then exit;

  VCTemp := TXYpointList.Create;

  If OperationMode = tbzSmooth Then
    k := 0 // set coefficients for  Exact modus
  Else
    k := 10; // set coefficients for  Exact modus

  If Closed Then // Handle the Start between first and last point
    With Vc Do Begin
      sc := sqrt(sqr(Points[2].x - Points[1].x) + sqr(Points[2].y -
        Points[1].y));
      If sc = 0 Then sc := 1;
      step := PointStep / sc;
      l := 0;
      Repeat
        For j := 0 To 3 Do Begin
          x := Bezier(l, k) * Points[MaxPoint - 1].x
            + Bezier(l, k + 1) * Points[1].x
            + Bezier(l, k + 2) * Points[2].x
            + Bezier(l, k + 3) * Points[3].x;
          y := Bezier(l, k) * Points[MaxPoint - 1].y
            + Bezier(l, k + 1) * Points[1].y
            + Bezier(l, k + 2) * Points[2].y
            + Bezier(l, k + 3) * Points[3].y;
        End; // end For
        Point := TXYPoint.Create;
        Point.X := X; Point.Y := Y;
        VCTemp.Add(Point);
        l := l + step;
      Until l > 1;
    End; // end with

  // Handle the inner sections index 2 .. Max-2
  With Vc Do
    For i := 2 To MaxPoint - 2 Do Begin
      sc := sqrt(sqr(Points[i + 1].x - Points[i].x) + sqr(Points[i + 1].y
        - Points[i].y));
      If sc = 0 Then sc := 1;
      step := PointStep / sc;
      l := 0;
      Repeat
        x := Bezier(l, k) * Points[i - 1].x
          + Bezier(l, k + 1) * Points[i].x
          + Bezier(l, k + 2) * Points[i + 1].x
          + Bezier(l, k + 3) * Points[i + 2].x;
        y := Bezier(l, k) * Points[i - 1].y
          + Bezier(l, k + 1) * Points[i].y
          + Bezier(l, k + 2) * Points[i + 1].y
          + Bezier(l, k + 3) * Points[i + 2].y;
        Point := TXYPoint.Create;
        Point.X := X; Point.Y := Y;
        VCTemp.Add(Point);
        l := l + step;
      Until l > 1;
    End;

  // Handle the last points
  If Closed Then
    With Vc Do Begin
      sc := sqrt(sqr(Points[MaxPoint].x - Points[MaxPoint - 1].x)
        + sqr(Points[MaxPoint].y - Points[MaxPoint - 1].y));
      If sc = 0 Then sc := 1;
      step := PointStep / sc;
      l := 0;
      Repeat
        For j := 0 To 3 Do Begin
          x := Bezier(l, k) * Points[MaxPoint - 2].x
            + Bezier(l, k + 1) * Points[MaxPoint - 1].x
            + Bezier(l, k + 2) * Points[MaxPoint].x
            + Bezier(l, k + 3) * Points[2].x;
          y := Bezier(l, k) * Points[MaxPoint - 2].y
            + Bezier(l, k + 1) * Points[MaxPoint - 1].y
            + Bezier(l, k + 2) * Points[MaxPoint].y
            + Bezier(l, k + 3) * Points[2].y;
        End;
        Point := TXYPoint.Create;
        Point.X := X; Point.Y := Y;
        VCTemp.Add(Point);
        l := l + step;
      Until l > 1;
    End;
  Vc.Clear;
  Vc.Assign(VcTemp);
  VcTemp.Destroy;
End;
//--------------------------------------------------------------------------
End.
//--------------------------------------------------------------------------

