//------------------------------------------------------------------------------
// Testprogram Bezier Splining 3 order for polygons
//  - XII/2001 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------

{
@abstract(Formular Test Program Bezier Splining 3 order for polygons.)
@author  (Alexander Weidauer alex.weidauer@huckfinn.de)
@created (December 2001)
@lastmod (December 2001)
The Unit  delivers  the basic form for a test ptogram a smoothed bezier
spline of a polygon. }
Unit UTestBezier;
//------------------------------------------------------------------------------

Interface
//------------------------------------------------------------------------------

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  ExtCtrls,
  ComCtrls,
  ToolWin,
  uConst,
  uXYPoint,
  uXYPointList,
  UBezierSpline,
  StdCtrls;

//------------------------------------------------------------------------------
Type
  { Mainwindow of the test class }
  TMainWindow = Class(TForm)
    { Command Bar }
    tbCommand: TToolBar;
    { Button to create objects in smooth mode.}
    tbtSmoot: TToolButton;
    { Button to create objects in exact mode.}
    tbtExact: TToolButton;
    { Scrollbox for viewport.}
    sbMain: TScrollBox;
    { Paintbox for the viewport. }
    pbMain: TPaintBox;
    { 1. Button splitter }
    tbtSplit1: TToolButton;
    { 2. Button splitter }
    tbtSplit2: TToolButton;
    { Handle circle as mode rectangle }
    rbCircle1: TRadioButton;
    { Handle circle as mode rhombus }
    rbCircle2: TRadioButton;
    { Handle Ellipse }
    rbEllipse: TRadioButton;
    { Draw freestyle }
    rbFreestyle: TRadioButton;
    { Close polygons }
    cbClosed: TCheckBox;
    { Initialize something }
    Procedure FormCreate(Sender: TObject);
    { Handle the paint and redraw event }
    Procedure pbMainPaint(Sender: TObject);
    { Create the "smooth" beziers }
    Procedure tbtSmootClick(Sender: TObject);
    { Create the "exact" beziers }
    Procedure tbtExactClick(Sender: TObject);
  Private

    fTRFM: TAffinTransform;
    fMin, fMax: TXYPoint;
    fDrawMemory: TBitmap;
    fPolygon: TXYPointList;
    fSmoothPolygon: TXYPointList;
  Public
    { Create a image of the modell. }
    Procedure DrawPolygon;
    { Create some ways to generate the polygon. }
    Procedure GeneratePolygon;
  End;

Var
  MainWindow: TMainWindow;
//------------------------------------------------------------------------------

Implementation

{$R *.DFM}

Const cStep = 2;
//------------------------------------------------------------------------------
Procedure TMainWindow.FormCreate(Sender: TObject);
Begin
  fDrawMemory := TBitmap.Create;
  pbMain.Width := Screen.Width;
  pbMain.Height := Screen.Height;
  fDrawMemory.Width := Screen.Width;
  fDrawMemory.Height := Screen.Height;
  Show; BringToFront;
  fPolygon := TXYPointList.Create;
  fSmoothPolygon := TXYPointList.Create;
  With fTrfm Do Begin
    eM11 := 100; eM12 := 0;
    eM21 := 0; eM22 := 100;
    eDx := 110; eDy := 110;
  End;
  fMin := TXYPoint.Create;
  fMax := TXYPoint.Create;
End;

//------------------------------------------------------------------------------

Procedure TMainWindow.pbMainPaint(Sender: TObject);
Begin
  pbMain.Canvas.Draw(0, 0, fDrawMemory);
End;

//------------------------------------------------------------------------------

Procedure TMainWindow.GeneratePolygon;
Var i: Integer;
Begin
  fPolygon.Clear;
  fSmoothPolygon.Clear;

  If rbCircle1.Checked Then Begin
    fPolygon.Add(TXYPoint.CreateByCoord(-1, -1));
    fPolygon.Add(TXYPoint.CreateByCoord(1, -1));
    fPolygon.Add(TXYPoint.CreateByCoord(1, 1));
    fPolygon.Add(TXYPoint.CreateByCoord(-1, 1));
    If cbClosed.Checked Then
      fPolygon.Add(TXYPoint.CreateByCoord(-1, -1));
  End;

  If rbCircle2.Checked Then Begin
    fPolygon.Add(TXYPoint.CreateByCoord(0, 1));
    fPolygon.Add(TXYPoint.CreateByCoord(1, 0));
    fPolygon.Add(TXYPoint.CreateByCoord(0, -1));
    fPolygon.Add(TXYPoint.CreateByCoord(-1, 0));
    If cbClosed.Checked Then
      fPolygon.Add(TXYPoint.CreateByCoord(0, 1));
  End;

  If rbEllipse.Checked Then Begin
    fPolygon.Add(TXYPoint.CreateByCoord(-0.5, -1));
    fPolygon.Add(TXYPoint.CreateByCoord(0.5, -1));
    fPolygon.Add(TXYPoint.CreateByCoord(0.5, 1));
    fPolygon.Add(TXYPoint.CreateByCoord(-0.5, 1));
    If cbClosed.Checked Then
      fPolygon.Add(TXYPoint.CreateByCoord(-0.5, -1));
  End;

  If rbFreestyle.Checked Then Begin
    For i := 1 To 10 Do
      fPolygon.Add(TXYPoint.CreateByCoord(Random(100) / 50 - 0.5,
        Random(100) / 50 - 0.5));
    If cbClosed.Checked Then
      fPolygon.Add(TXYPoint.CreateByPoint(fPolygon.Points[1]));
  End;

  fPolygon.Transform(fTrfm, fMin, fMax);
  fSmoothPolygon.Assign(fPolygon);
End;
//------------------------------------------------------------------------------

Procedure TMainWindow.DrawPolygon;
Var i: Integer; x, y: Integer;
Begin
  With fDrawMemory.Canvas Do Begin
    Brush.Color := ClWhite;
    Brush.Style := bsSolid;
    FillRect(Cliprect);
    Pen.Color := clYellow;
    Brush.Style := bsSolid;
    For i := 1 To fPolygon.MaxPoint Do Begin
      x := Round(fPolygon.Points[i].X);
      Y := Round(fPolygon.Points[i].y);
      Brush.Color := ClWhite;
      TextOut(x, y, '  ' + IntToStr(i));
      Brush.Color := ClBlack;
      Rectangle(x - 5, y - 5, x + 5, y + 5);
    End;
    Pen.Color := clRed;
    Brush.Color := clWhite;
    Brush.Style := bsClear;
    For i := 1 To fSmoothPolygon.MaxPoint Do Begin
      x := Round(fSmoothPolygon.Points[i].X);
      Y := Round(fSmoothPolygon.Points[i].y);
      If i = 1 Then MoveTo(x, y) Else lineto(x, y);
    End;
  End;
  pbMain.Invalidate
End;
//------------------------------------------------------------------------------

Procedure TMainWindow.tbtSmootClick(Sender: TObject);
Begin
  GeneratePolygon;
  BezierPolygon(tbzSmooth, cbClosed.Checked, fSmoothPolygon, 2);
  DrawPolygon;
End;
//------------------------------------------------------------------------------

Procedure TMainWindow.tbtExactClick(Sender: TObject);
Begin
  generatePolygon;
  BezierPolygon(tbzExact, cbClosed.Checked, fSmoothPolygon, cStep);
  DrawPolygon;
End;
//------------------------------------------------------------------------------
End.
//------------------------------------------------------------------------------

