//------------------------------------------------------------------------------
// XYPointContainer for Vector Objects
//  - II/2002 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------

{
@abstract(Representation for a 2 dimensional pointobject.)
@author  (Alexander Weidauer alex.weidauer@huckfinn.de)
@created (February 2002)
@lastmod (February 2002)
The Unit  delivers the modell for a 2 dimensional point object as well as
some service routines to compare and transform the object geomtrically.
}
Unit UXYPoint;
//------------------------------------------------------------------------------
Interface
//------------------------------------------------------------------------------
Uses Windows,   // Window API Capsulation
  SysUtils,     // Service Routines Delphi
  Classes,      // Base Classes FCL/VCL Library Delphi
  Math,         // Mathmatical Routines
  UErrors,      // IfGDV ErrorHandler
  UConst,       // Base type declaration
  UXYIntersect,  // Intersection of 2 dimen. segments
  UStringUtils; // Stringroutine and Parser

//------------------------------------------------------------------------------
Const
 { String constant for the  token in interpreting modus. }
  CXYPointTk = 'TXYPoint';
 { String formatter for the  data in interpreting modus. }
  CXYPoint = ' (%E,%E)';
 { String formatter for the data and token  in interpreting modus. }
  CXYPointTkn = 'Point=(%E,%E)';
//------------------------------------------------------------------------------
Type
//------------------------------------------------------------------------------
 { Container for the affine transformation }
  TAffinTransform = Record
    eM11, eM21, eM12, eM22,
      eDX, eDY: TDouble;
  End;
//------------------------------------------------------------------------------
 {Definition of the 2 dimensionsl point object with double prec. }
  TXYPoint = Class(TObject)
  Private
             {Internal x-Coordinate (Rechtswert) west to east. }
    fX: TDouble;
             {Internal y-Cordinate  (Hochwert) soutrh to nort. }
    fY: TDouble;
  Public
             { Standard constructor. }
    Constructor Create;
             { Constructor with given point coordinates. }
    Constructor CreateByCoord(aX, aY: TDouble);
             { Construktor by given point (as a copy). }
    Constructor CreateByPoint(aPoint: TXYPoint);
             { Standard destructor. }
    Destructor Destroy; Override;
             { Point class to string conversation with formatting string
               defined by CONST CXYPointTkn. }
    Function ToStr: String;
             { Make a copy of the data of X. }
    Procedure Assign(aPoint: TXYPoint);
             { Binary writing. }
    Procedure Write(Port: TWriter);
             { Binary reading. }
    Procedure Read(Port: TReader);
             { Textual writing.  }
    Procedure List(Port: TStrings);
             { Affine transformation. }
    Procedure Transform(TRFM: TAffinTransform);
             { Read from a string defined by CONST CXYPointTkn.
               EofLine search for ';' after the string else search for ','}
    Function ReadASC(Var Line: String; EofLine: TBoolean): TBoolean;
             { Read from a string defined by %S=(%E,%E).
               EofLine search for ';' after the string else search for ','}
    Function ReadASCTol(Var Line: String; EofLine: TBoolean): TBoolean;
             { Schreiben in einen String mit der Formatierung %S=(%E,%E)
               EofLine will close for ';' after the string else close with ','}
    Function WriteASC(Line: String; EofLine: TBoolean): String;

             {The x-Coordinate (Rechtswert) west to east. }
    Property X:TDouble Read fX Write fX;
             {The y-Cordinate  (Hochwert) soutrh to nort. }
    Property Y:TDouble Read fY Write fY;
  End;
//------------------------------------------------------------------------------
 { Synonym for TXYPoint }
  TXYVector = TXYPoint;


//------------------------------------------------------------------------------
// Functional declaraitions
//------------------------------------------------------------------------------

 { Comparision of two points. }
Function XYPointEqual(P1, P2: TXYPoint): TBoolean;

//------------------------------------------------------------------------------

 { Comparision of two points with tolerance.
          ExXYPointEqual:=((Round(P1.X*Times)=Round(P2.X*Times))
                        AND(Round(P1.Y*Times)=Round(P2.Y*Times)));
 }
Function ExXYPointEqual(Times: TDouble; P1, P2: TXYPoint): TBoolean;

//------------------------------------------------------------------------------

 { Coordinat swapping from P1 to P2 and also in opposit direction. }
Procedure SwapXYPoint(P1, P2: TXYPoint);

//------------------------------------------------------------------------------

 { Initialization of the mini-/maximal extention by opposit quatities .}
Procedure XYPointInitMinMax(aMin, aMax: TXYPoint);

//------------------------------------------------------------------------------

 { Update the  mini-/maximal Extention with the values of aPoint and stor it
   in aMin,aMax. }
Procedure XYPointMinMax(aPoint: TXYPoint; aMin, aMax: TXYPoint);

//------------------------------------------------------------------------------

 { Query the location of aPoint in the rectangle defined by (aMin,aMax).
   IF aPoint is inside the function gives back TRUE. }
Function IsInside(aPoint: TXYPoint; aMin, aMax: TXYPoint): TBoolean;

//------------------------------------------------------------------------------

 { Sorting of the extremal points of the rectangle defined by aMin,aMax. }
Procedure SortRect(aMin, aMax: TXYPoint);

//------------------------------------------------------------------------------

 { Size of the rectangle defined by aMin,aMax. }
Function SizeRect(aMin, aMax: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Query the overlapping of the rectangels Rect1 and Rect 2 defined by
   Rect1Min,Rect1aMax and Rect2Min,Rect2Max. If they are not disjucnt the
   function gives back TRUE }
Function OverlapRect(Rect1Min, Rect1Max, Rect2Min, Rect2Max: TXYPoint):
  TBoolean;

//------------------------------------------------------------------------------

 { Query the overlapping of the Rectangels Rect1 and Rect 2 defined by
   Rect1MinX,Rect1MinY,Rect1MaxX,Rect1MaxY and
   Rect2MinX,Rect2MinY,Rect2MaxX,Rect2MaxY.
   If they are not disjucnt thefunction gives back TRUE }
Function OverlapRectCoord(Rect1MinX, Rect1MinY,
  Rect1MaxX, Rect1MaxY,
  Rect2MinX, Rect2MinY,
  Rect2MaxX, Rect2MaxY: TDouble): TBoolean;

//------------------------------------------------------------------------------

 { Euclidian distance of 2 points }
Function XYLength(A, B: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Euclidian distance coordinate wise of 2 points }
Function XYLengthCoord(X1, Y1, X2, Y2: TDouble): TDouble;

//------------------------------------------------------------------------------

 { Square of euclidian distance of 2 points }
Function XYSQRLength(A, B: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Area of the inclosed triangle with following orientation.
   Positiv areal values cw orientation.
   Negativ areal values ccw orientation.  }
Function TriXYArea(PBegin, PMiddle, PEnd: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Cosine between the three points. You can not see the orientation (hafplane).
   If the area of the inclosed triangle is zero the the functio gives back
   1E38.}
Function TriXYCos(PBegin, PMiddle, PEnd: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Angle between the three points in deg that means 0-360.
   If the area of the inclosed triangle is zero the the functio gives back
   1E38.}
Function TriXYAngle(PBegin, PMiddle, PEnd: TXYPoint): TDouble;

//------------------------------------------------------------------------------

 { Reading routine by given separators for plain coordinates.
   Formatting XXXXXS1YYYYYS2. If somting is wrong the function will
   be false. Please call the string GetError from Unit UStringutils.}
Function ReadPPoint(S1, S2: Char; Var S: String; P: TXYPoint): TBoolean;
//------------------------------------------------------------------------------

 { Reading routine by given separators for decdeg coordinates.
   Formatting XXXXXS1YYYYYS2. If somting is wrong the function will
   be false. Please call the string GetError from Unit UStringutils.}
Function ReadPPointDeg(S1, S2: Char; Var S: String; P: TXYPoint): TBoolean;
//------------------------------------------------------------------------------

{ Intersecting Segments P1P2 and Q1Q2. Result is NOT NIL if exists and
  if it is paralell IsPararlell is  TRUE. }
Function IntersectPoints(P1,P2,Q1,Q2:TXYPoint;Var IsParalell:TBoolean):TXYPoint;


Var DummyXYPoint:TXYPoint;
//------------------------------------------------------------------------------
Implementation

//------------------------------------------------------------------------------

Function IntersectPoints(P1,P2,Q1,Q2:TXYPoint;Var IsParalell:TBoolean):TXYPoint;
 Var X,Y:Double;
 Begin
  Result:=NIL;
  If ISec(P1.x,P1.y,P2.x,P2.y,Q1.x,Q1.y,Q2.x,Q2.y,X,Y,IsParalell) Then
   Result:=TXYPoint.CreateByCoord(X,Y);
 End;
//------------------------------------------------------------------------------


Function ReadPPoint(S1, S2: Char; Var S: String; P: TXYPoint): TBoolean;
Var X, Y: TDouble; D1, D2: TBoolean;
Begin
  D1 := ReadPDouble(S1, S, X);
  D2 := ReadPDouble(S2, S, Y);
  Result := D1 And D2;
  If Not D1 Then SetError('Reading TDouble MM Point X');
  If Not D2 Then SetError('Reading TDouble MM Point Y');
  P.X := X;
  P.Y := Y;
End;
//------------------------------------------------------------------------------

Function ReadPPointDeg(S1, S2: Char; Var S: String; P: TXYPoint): TBoolean;
Var X, Y: TDouble; D1, D2: TBoolean;
Begin
  D1 := ReadPDegToDecDeg(S1, S, X);
  D2 := ReadPDegToDecDeg(S2, S, Y);
  Result := D1 And D2;
  If Not D1 Then SetError('Reading TDouble DEG Point X');
  If Not D2 Then SetError('Reading TDouble DEG Point Y');
  P.X := X;
  P.Y := Y;
End;
//------------------------------------------------------------------------------

Function XYPointEqual(P1, P2: TXYPoint): TBoolean;
Begin XYPointEqual := ((P1.X = P2.X) And (P1.Y = P2.Y)); End;
//------------------------------------------------------------------------------

Function ExXYPointEqual(Times: TDouble; P1, P2: TXYPoint): TBoolean;
Begin ExXYPointEqual := ((Round(P1.X * Times) = Round(P2.X * Times))
    And (Round(P1.Y * Times) = Round(P2.Y * Times))); End;
//------------------------------------------------------------------------------

Function XYLength(A, B: TXYPoint): TDouble;
Begin
  XYLength := SQRT(SQR(A.X - B.X) + SQR(A.Y - B.Y));
End;
//------------------------------------------------------------------------------

Function XYLengthCoord(X1, Y1, X2, Y2: TDouble): TDouble;
Begin
  XYLengthCoord := SQRT(SQR(X1 - X2) + SQR(Y1 - Y2));
End;
//------------------------------------------------------------------------------

Procedure XYPointInitMinMax(aMin, aMax: TXYPoint);
Begin
  aMin.X := MaxDouble;
  aMin.Y := MaxDouble;
  aMax.X := -MaxDouble;
  aMax.Y := -MaxDouble;
End;
//------------------------------------------------------------------------------

Procedure XYPointMinMax(aPoint: TXYPoint; aMin, aMax: TXYPoint);
Begin
  If aPoint.X < aMin.X Then aMin.X := aPoint.X;
  If aPoint.Y < aMin.Y Then aMin.Y := aPoint.Y;
  If aPoint.X > aMax.X Then aMax.X := aPoint.X;
  If aPoint.Y > aMax.Y Then aMax.Y := aPoint.Y;
End;
//------------------------------------------------------------------------------

Function OverLapRect(Rect1Min, Rect1Max, Rect2Min, Rect2Max: TXYPoint):
  TBoolean;
Begin
  OverlapRect := False;
  SortRect(Rect1Min, Rect1Max);
  SortRect(Rect2Min, Rect2Max);
  If (Rect1Min.X >= Rect2Max.X) Or (Rect1Max.X <= Rect2Min.X) Or
    (Rect1Min.Y >= Rect2Max.Y) Or (Rect1Max.Y <= Rect2Min.Y) Then Exit;
  OverlapRect := True;
End;
//------------------------------------------------------------------------------

Function OverlapRectCoord(Rect1MinX, Rect1MinY,
  Rect1MaxX, Rect1MaxY,
  Rect2MinX, Rect2MinY,
  Rect2MaxX, Rect2MaxY: TDouble): TBoolean;
Begin
  OverlapRectCoord := False;
  If (Rect1MinX >= Rect2MaxX) Or (Rect1MaxX <= Rect2MinX) Or
    (Rect1MinY >= Rect2MaxY) Or (Rect1MaxY <= Rect2MinY) Then Exit;
  OverlapRectCoord := True;
End;
//------------------------------------------------------------------------------

Procedure SortRect(aMin, aMax: TXYPoint);
Var Helper: TDouble;
Begin
  If aMin.X > aMax.X Then
    Begin Helper := aMin.X; aMin.X := aMax.X; aMax.X := Helper; End;
  If aMin.Y > aMax.Y Then
    Begin Helper := aMin.Y; aMin.Y := aMax.Y; aMax.Y := Helper; End;
End;
//------------------------------------------------------------------------------

Function SizeRect(aMin, aMax: TXYPoint): TDouble;
Begin
  SortRect(aMin, aMax);
  Result := (aMax.X - aMin.X) * aMax.Y - aMin.Y;
End;
//------------------------------------------------------------------------------

Function XYSQRLength(A, B: TXYPoint): TDouble;
Begin
  XYSQRLength := SQR(A.X - B.X) + SQR(A.Y - B.Y);
End;
//------------------------------------------------------------------------------

Function TriXYAngle(PBegin, PMiddle, PEnd: TXYPoint): TDouble;
Var AC, ACX, ACG, TR: TDouble;
Begin
  TriXYAngle := 1E38;
  TR := TriXYArea(PBegin, PMiddle, PEnd);
  AC := TriXYCos(PBegin, PMiddle, PEnd);
  ACX := ArcCos(AC);
  ACG := ACX / Pi * 180;
  If Tr >= 0 Then TriXYAngle := +ACG;
  If Tr < 0 Then TriXYAngle := 360 - ACG;
  If AC = 1E38 Then TriXYAngle := 1E38;
End;
//------------------------------------------------------------------------------

Function TriXYArea(PBegin, PMiddle, PEnd: TXYPoint): TDouble;
Begin
  TriXYArea := (PBegin.y + PMiddle.y) / 2 * (PMiddle.x - PBegin.X)
    + (PMiddle.y + PEnd.y) / 2 * (PEnd.x - PMiddle.X)
    + (PBegin.y + PEnd.y) / 2 * (PBegin.x - PEnd.X);
End;
//------------------------------------------------------------------------------

Function XYNorm(V: TXYPoint): TDouble;
Begin
  XYNorm := V.X * V.X + V.Y * V.Y;
End;
//------------------------------------------------------------------------------

Function TriXYCos(PBegin, PMiddle, PEnd: TXYPoint): TDouble;
Var a, b, c: TDouble;
Begin
  a := XYLength(PBegin, PMiddle);
  b := XYLength(PMiddle, PEnd);
  c := XYLength(PEnd, PBegin);
  If a * b = 0 Then
    TriXYCos := 1E38
  Else
    TriXYCos := (SQR(C) - SQR(A) - SQR(B)) / -(2 * A * B); // Cosinussatz
End;
//------------------------------------------------------------------------------

Function IsInside(aPoint: TXYPoint; aMin, aMax: TXYPoint): TBoolean;
Begin
  IsInside :=
    ((APoint.X >= aMin.X) And (APoint.X <= aMax.X) And
    (APoint.Y >= aMin.Y) And (APoint.Y <= aMax.Y));
End;
//------------------------------------------------------------------------------

Procedure SwapXYPoint(P1, P2: TXYPoint);
Var X, Y: TDouble;
Begin
  X := P1.X; Y := P1.Y;
  P1.Y := P2.Y; P1.X := P2.X;
  P2.X := X; P2.Y := Y;
End;

//------------------------------------------------------------------------------
// Declaration of TXYPoint
//------------------------------------------------------------------------------

Constructor TXYPoint.Create;
Begin
  Inherited Create;
  DecimalSeparator := '.';
  fX := 0; fY := 0;
End;
//------------------------------------------------------------------------------

Constructor TXYPoint.CreateByCoord(aX, aY: TDouble);
Begin
  Inherited Create;
  DecimalSeparator := '.';
  fX := aX; fY := aY;
End;
//------------------------------------------------------------------------------

Constructor TXYPoint.CreateByPoint(aPoint: TXYPoint);
Begin
  Inherited Create;
  DecimalSeparator := '.';
  fX := aPoint.X;
  fY := aPoint.Y;
End;
//------------------------------------------------------------------------------

Destructor TXYPoint.Destroy;
Begin
  fx := 0; fy := 0;
  Inherited Destroy;
End;
//------------------------------------------------------------------------------

Procedure TXYPoint.Write(Port: TWriter);
Begin
  Port.Write(fx, SizeOf(fx));
  Port.Write(fy, SizeOf(fy));
End;
//------------------------------------------------------------------------------

Procedure TXYPoint.Read(Port: TReader);
Begin
  Port.Read(fx, SizeOf(fx));
  Port.Read(fy, SizeOf(fy));
End;
//------------------------------------------------------------------------------

Procedure TXYPoint.Assign(aPoint: TXYPoint);
Begin
  fX := aPoint.X; fY := aPoint.Y;
End;
//------------------------------------------------------------------------------

Function TXYPoint.ToStr: String;
Begin ToStr := Format(CXYPoint, [fX, fY]); End;
//------------------------------------------------------------------------------

Procedure TXYPoint.Transform(TRFM: TAffinTransform);
Var tx, ty: TDouble;
Begin
  tx := x; ty := y;
  x := TRFM.eM11 * tx + TRFM.eM12 * Ty + TRFM.eDx;
  y := TRFM.eM21 * tx + TRFM.eM22 * Ty + TRFM.eDy;
End;
//------------------------------------------------------------------------------

Function TXYPoint.WriteASC(Line: String; EofLine: TBoolean): String;
Var S: String;
Begin
  S := Format(CXYPoint, [fX, fY]);
  If EofLine Then Line := Line + S + ';'
  Else Line := Line + S + ',';
  WriteASC := Line;
End;
//------------------------------------------------------------------------------

Function TXYPoint.ReadASC(Var Line: String; EofLine: TBoolean): TBoolean;
Var H, Data: String; Done: TBoolean;
Begin

  REadASC := False;
  If Not ReadPStr(TheEqual, Line, H) Then Begin
    SetError(CLoadError + CXYPoint + CNewLine + GetError);
    Exit;
  End;

  If Not (Uppercase(H) = 'POINT') Then Begin
    SetError(geterror + 'POINT=()' + CRequiered + CNewLine);
    Exit;
  End;

  If EofLine Then Done := ReadBracedString(TheTerminator, Line, Data)
  Else Done := ReadBracedString(TheSeparator, Line, Data);

  If Not Done Then Begin
    SetError(CLoadError + CXYPoint + CNewLine + GetError);
    Exit;
  End;

  Data := Data + ';';
  ReadASC := ReadPDouble(TheSeparator, Data, fX) And
    ReadPDouble(')', Data, fY);

  If Not Done Then Begin
    SetError(CLoadError + CXYPoint + CNewLine + GetError);
    Exit;
  End;
  ReadASC := True;

End;
//------------------------------------------------------------------------------

Function TXYPoint.ReadASCTol(Var Line: String; EofLine: TBoolean): TBoolean;
Var H, Data: String; Done: TBoolean;
Begin

  ReadASCTol := False;
  ReadPStr(TheEqual, Line, H);

  If EofLine Then Done := ReadBracedString(TheTerminator, Line, Data)
  Else Done := ReadBracedString(TheSeparator, Line, Data);

  If Not Done Then Begin
    SetError(CLoadError + CXYPoint + CNewLine + GetError);
    Exit;
  End;

  Data := Data + ';';
  ReadASCTol := ReadPDouble(TheSeparator, Data, fX) And
    ReadPDouble(')', Data, fY);

  If Not Done Then Begin
    SetError(CLoadError + CXYPoint + CNewLine + GetError);
    Exit;
  End;
  ReadASCTol := True;

End;
//------------------------------------------------------------------------------

Procedure TXYPoint.List(Port: TStrings);
Begin
  Port.Add(Format('Point=(%E,%E)', [fx, fy]));
End;
//------------------------------------------------------------------------------

Initialization
 DummyXYPoint:=TXYPoint.Create;
End.
//------------------------------------------------------------------------------

