//------------------------------------------------------------------------------
// XYPointListContainer for Vector Objects
//  - XII/2001 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------

{
@abstract(Representation for a 2 dimensional polygon object.)
@author  (Alexander Weidauer alex.weidauer@huckfinn.de)
@created (December 2001)
@lastmod (December 2001)
The Unit  delivers the modell for a 2 dimensional polygon object as well as
some service routines to compare and transform the object geomtrically.}
Unit UXYPointList;
//------------------------------------------------------------------------------
Interface
//------------------------------------------------------------------------------
Uses Windows, // Window API Capsulation
  SysUtils, // Service Routines Delphi
  Classes, // Base Classes FCL/VLC Library Delphi
  Math, // Mathmatical Routines
  UConst, // Type and Constatntdeclarations
  UErrors, // IfGDV ErrorHandler
  UStringUtils, // Strinroutine and Parser
  UXYPoint; // Base Container for GeoPoints
//------------------------------------------------------------------------------
Const
{ String constant for the  token in interpreting modus. }
  CXYPointListTK = 'TXYPointList';

//------------------------------------------------------------------------------
Type

{ Error constants for some checks:
    xyplETOk - Everything is fine.
    xyplETSelfCutting - Polygon cuts themself.
    xyplETHangingNodes - Polygon has hanging nodes.
    xyplETDoubleNodes - Polygon has TDouble nodes.
    xyplETPoorNodes - Polygon is poor formed (0 or 1 element).
}
  TXYPointListTopError = (xyplETOk,
    xyplETSelfCutting,
    xyplETHangingNodes,
    xyplETDoubleNodes,
    xyplETPoorNodes,
    xyplET_NIL);

 {Definition of the 2 dimensionsl polygon object with TDouble prec. }
  TXYPointList = Class(TObject)
  Private
     {internal counter for number of points}
    fMaxPoint :TInt32;
                 { The point container will be emmbedded linear
                   by the properties. }
    fData     :TList;
     { flag for clean topography }
    fClean    :TBoolean;
  Public
                 { Standard constructor }
    Constructor Create;
                 { Standard constructor }
    Constructor CreateByPoints(aPointList: TXYPointList);
                 { Standard desstructor }
    Destructor Destroy; Override;
                 { Clear the current list }
    Procedure Clear;
                 { Append a point at the end of the current list }
    Procedure Add(aPoint: TXYPoint);
                 { Insert a point at the index of the current list }
    Procedure Insert(Index: TInt32; aPoint: TXYPoint);
                 { Delete a point at the index of the current list }
    Procedure Delete(Index: TInt32);
                 { Substitute a point at the index of the current list }
    Procedure Put(Index: TInt32; aPoint: TXYPoint); Virtual;
                 { Get the data of point at the index of the current list }
    Function  Get(Index: TInt32): TXYPoint; Virtual;
                 { Extent of the Polygon. }
    Procedure Extent(aMin, aMax: TXYPoint);
                 { Affine transformation of a point and recalculation of it's
                   extention }
    Procedure Transform(Trfm: TAffinTransform;
      Var aMin, aMax: TXYPoint);
                 { Length of a polygon. }
    Function PolyLength: TDouble;
                 { Binary writing of the polygon }
    Procedure Write(Port: TWriter);
                 { Binary reading of the polygon }
    Procedure Read(Port: TReader);
                 { Textrual writing  to a stringlist.}
    Procedure List(Port: TStrings);
                 { Position of the geometrical center (in german Schwerpunkt). }
    Function HeavyPoint: TXYPoint;
                 { Create a copy of aPolygon }
    Procedure Assign(aPointList: TXYPointList);
                 { Check topographical errors. }
    Function CheckTopography(Var IndexA,IndexB,IndexC:TInt32):TXYPointListTopError;

                 { Linear encapsulation of the polygonset.
                   The index starts with 1 and ends with Maxpoints. }
    Property Points[index: TInt32]: TXYPoint Read Get Write Put;
                 { Number of elements of a Polygonset. }
    Property MaxPoint: TInt32 Read fMaxPoint;
  End;
//------------------------------------------------------------------------------
Implementation
//------------------------------------------------------------------------------
Uses UXYLine,
  UXYLineTable;
//------------------------------------------------------------------------------
Constructor TXYPointList.Create;
Begin
  Inherited Create;
  fData := TList.Create;
  fMaxPoint:=0;
  fClean:=False;
End;
//------------------------------------------------------------------------------
Constructor TXYPointList.CreateByPoints(aPointList: TXYPointList);
Begin
  Inherited Create;
  fData := TList.Create;
  Assign(aPointList);
  fClean:=False;
End;
//------------------------------------------------------------------------------

Destructor TXYPointList.Destroy;
Var i: TInt32;
Begin
  Clear;
  fData.Destroy;
  Inherited Destroy;
End;
//------------------------------------------------------------------------------
Procedure TXYPointList.Write(Port: TWriter);
Var i: TInt32;
Begin
  i := MaxPoint;
  Port.Write(i, SizeOf(i));
  For i := 1 To MaxPoint Do Points[i].Write(Port);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Read(Port: TReader);
Var i, m: TInt32;
Begin
  Clear;
  Port.Read(m, SizeOf(m));
  For i := 1 To m Do Begin
    Add(TXYPoint.Create);
    Points[MaxPoint].Read(Port);
  End;
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Assign(aPointList: TXYPointList);
Var i, m: TInt32;
Begin
  Clear;
  m := aPointList.MaxPoint;
  For i := 1 To m Do Add(TXYPoint.CreateByPoint(aPointList.Points[i]));
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Add(aPoint: TXYPoint);
Begin
  fData.Add(aPoint);Inc(fMaxPoint); fClean:=False;
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Insert(Index: TInt32; aPoint: TXYPoint);
Begin
  fData.Insert(Index - 1, aPoint); fClean:=False; Inc(fMaxPoint);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Delete(Index: TInt32);
Begin
  TXYPoint(fData.Items[index - 1]).Free;
  fData.Delete(Index - 1);dec(fMaxPoint);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Put(Index: TInt32; aPoint: TXYPoint);
Begin
  TXYPoint(fData.Items[Index - 1]).Assign(aPoint);fClean:=False;

End;
//------------------------------------------------------------------------------

Function TXYPointList.Get(Index: TInt32): TXYPoint;
Begin
  REsult := Nil; Result := TXYPoint(fData.Items[Index - 1]);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Clear;
Var i: TInt32;
Begin
  For i := 1 To fMaxPoint Do Begin
    TXYPoint(fData.Items[i - 1]).Free;
    fData.Items[i - 1] := Nil;
  End;
  fData.Pack; fData.Clear; fClean:=False;fMaxPoint:=0;
End;
//------------------------------------------------------------------------------

Function TXYPointList.PolyLength: TDouble;
Var i: TInt32; Sum: TDouble;
Begin
  Sum := 0;
  For i := 1 To MaxPoint - 1 Do Begin
    sum := sum + XYLength(Points[i], Points[i + 1]);
  End;
  PolyLength := SUM;
End;
//------------------------------------------------------------------------------

Function TXYPointList.HeavyPoint: TXYPoint;
Var X, Y: TDouble; i: TInt32;
Begin
  X := 0; Y := 0;
  If MaxPoint = 0 Then Exit;
  For i := 1 To MaxPoint Do Begin
    X := X + Points[i].X;
    Y := Y + Points[i].Y;
  End;
  X := X / MaxPoint;
  Y := Y / MaxPoint;
  Result := TXYPoint.CreateByCoord(X, Y);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Extent(aMin, aMax: TXYPoint);
Var j, m: TInt32;
Begin
  XYPointInitMinMax(aMin, aMax);
  If MaxPoint = 0 Then Exit;
  m := MaxPoint;
  For j := 1 To m Do XYPointMinMax(Points[j], aMin, aMax);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.Transform(TRFM: TAffinTransform; Var aMin, aMax:
  TXYPoint);
Var j, m: TInt32;
Begin
  If MaxPoint = 0 Then Exit;
  m := MaxPoint;
  For j := 1 To m Do Points[j].Transform(Trfm);
  Extent(aMin, aMax);
End;
//------------------------------------------------------------------------------

Procedure TXYPointList.List(Port: TStrings);
Var i: TInt32;
Begin
  Port.Add(Format('MaxPoint=%3d', [MaxPoint]));
  For i := 1 To MaxPoint Do Begin
    Port.Add(Format('Point[%3d]=(%E,%E)', [i, Points[i].X, Points[i].Y]));
  End;
  Port.Add('');
End;
//------------------------------------------------------------------------------

Function TXYPointList.CheckTopography(Var IndexA,IndexB,IndexC:TInt32): TXYPointListTopError;
Var i,j:TInt32;Para:TBoolean;
    aPoint:TXYPoint;
Begin
  Result := xyplET_NIL;

  IndexA:=-1;
  IndexB:=-1;
  IndexC:=-1;

  // not enough points
  If MaxPoint < 2 Then Begin Result := xyplETPoorNodes; Exit; End;

  // double following nodes
  For i:=1 to MaxPoint-1 Do
   If XYLength(Points[i],Points[i+1]) = 0 Then Begin
    Result := xyplETDoubleNodes;
    IndexA:=I;
    Exit;
   End;

  // hanging nodes
  For i:=2 to MaxPoint Do
   If XYPointEqual(Points[i-1],Points[i+1]) AND
      (TriXYArea(Points[i-1],Points[i],Points[i+1])=0) Then Begin
    IndexA:=i-1;
    IndexB:=i;
    IndexC:=i+1;
   End;

  // self cutting
  For i:=1 to MaxPoint-1 Do
   For j:=i+2 to MaxPoint-2 Do
    If i<>j Then Begin
     aPoint:=IntersectPoints(Points[i],Points[i+1],Points[j],Points[j+1],
     Para);
    If Not(aPoint=NIL) Then Begin
     Result := xyplETSelfCutting;
     IndexA:=I;
     IndexB:=J;
     aPoint.Free;
     Exit;
    End;
   End; 
 Result:=xyplETOk;
 fClean:=True;
End;

End.

