unit main;

// split by:
// lines == exact number of whole lines in each output file
// chars == max number of whole lines in each output file such that total
//          number of chars is not exceeded.  char count include CR/LF at
//          the end of each line
// bytes == exact number of bytes in each output file, regardless of lines
//          or if text or binary files

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, Buttons, FileCtrl, ComCtrls, Spin, ExtCtrls;

type
    TBuf128 = packed array[1..128] of byte;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    PageControl1: TPageControl;
    Tab2: TTabSheet;
    Tab1: TTabSheet;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    Edit1: TEdit;
    Radio1: TRadioButton;
    Radio2: TRadioButton;
    Radio3: TRadioButton;
    bSFile: TSpeedButton;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Edit3: TEdit;
    bSDir: TSpeedButton;
    Edit4: TEdit;
    Spin1: TSpinEdit;
    Check1: TCheckBox;
    Check2: TCheckBox;
    bSplit: TSpeedButton;
    bCancelSplit: TSpeedButton;
    GroupBox3: TGroupBox;
    Static1: TStaticText;
    GroupBox4: TGroupBox;
    Label6: TLabel;
    Edit2: TEdit;
    bMFile: TSpeedButton;
    OpenDialog2: TOpenDialog;
    Label7: TLabel;
    Label8: TLabel;
    Static2: TStaticText;
    Static3: TStaticText;
    Label9: TLabel;
    Label10: TLabel;
    GroupBox5: TGroupBox;
    Edit5: TEdit;
    Edit6: TEdit;
    Label11: TLabel;
    Label12: TLabel;
    bMDir: TSpeedButton;
    GroupBox6: TGroupBox;
    Static4: TStaticText;
    Label13: TLabel;
    Static5: TStaticText;
    bMerge: TSpeedButton;
    bCancelMerge: TSpeedButton;
    TabSheet1: TTabSheet;
    RTF1: TRichEdit;
    Image1: TImage;
    Image2: TImage;
    procedure FormCreate(Sender: TObject);
    procedure bSFileClick(Sender: TObject);
    procedure bSDirClick(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Check1Click(Sender: TObject);
    procedure bSplitClick(Sender: TObject);
    procedure bCancelSplitClick(Sender: TObject);
    procedure SplitLines;
    procedure SplitChars;
    procedure SplitBytes;
    function  Base36(aNumber: integer): string;
    procedure bMFileClick(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure bMDirClick(Sender: TObject);
    procedure bMergeClick(Sender: TObject);
    procedure bCancelMergeClick(Sender: TObject);

  private
    fHomeDir:           string;         // application home
    fCancel:            boolean;        // did use cancel?
    fIName:             string;         // input file name
    fOName:             string;         // output file name
    fNumber:            integer;        // number of split file
    fSize:              integer;        // number of lines/chars/bytes


  public
  end;

var
  Form1: TForm1;

{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }

implementation

{$R *.DFM}

{ ---------------------------------------------------------------------------- }

procedure TForm1.FormCreate(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin

// center on screen

Left := (Screen.Width  - Width)  div 2;
Top  := (Screen.Height - Height) div 2;

// save startup directory

fHomeDir := ExtractFilePath(Application.ExeName);
i := length(fHomeDir);
if ((i > 0) and (fHomeDir[i] <> '\')) then fHomeDir := fHomeDir + '\';

// set the real limits on the size select spin button

Spin1.MaxValue := 1024 * 1024;
Spin1.Value    := 1;
Spin1.MinValue := 1;

// start on the right page

PageControl1.ActivePage := Tab1;

// load the help file

s := fHomeDir + 'SplitFile.rtf';
if (FileExists(s)) then RTF1.Lines.LoadFromFile(s);
end;

{ ---------------------------------------------------------------------------- }
{ select a file to split }

procedure TForm1.bSFileClick(Sender: TObject);
begin
OpenDialog1.InitialDir := fHomeDir;
if (OpenDialog1.Execute) then
    begin
    Edit1.Text := OpenDialog1.Filename;
    end;
end;

procedure TForm1.Edit1Change(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin

// output file directory

s := ExtractFilePath(Edit1.Text);
i := length(s);
if ((i > 0) and (s[i] <> '\')) then s := s + '\';
Edit3.Text := s;

// output file prefix

s := ExtractFilename(Edit1.Text);
i := pos('.', s);
if (i > 0) then s[i] := '$';
Edit4.Text := s;

// we must have something to split now

bSplit.Enabled := true;
end;


{ ---------------------------------------------------------------------------- }
{ select destination directory }

procedure TForm1.bSDirClick(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin
if (SelectDirectory('Destination Directory', '', s)) then
    begin
    i := length(s);
    if ((i > 0) and (s[i] <> '\')) then s := s + '\';
    Edit3.Text := s;
    end;
end;

{ ---------------------------------------------------------------------------- }
{ select size modifier }

procedure TForm1.Check1Click(Sender: TObject);
var
    i,n:    integer;
    cb:     TCheckBox;
begin
cb := TCheckBox(Sender);
n  := cb.Tag;

if ((n = 1) and (Check1.Checked) and (Check2.Checked)) then Check2.Checked := false;
if ((n = 2) and (Check2.Checked) and (Check1.Checked)) then Check1.Checked := false;
end;

{ ---------------------------------------------------------------------------- }
{ split the input file into many output files }

procedure TForm1.bSplitClick(Sender: TObject);
var
    i,n:    integer;
begin

// save data from the form

fIName := Edit1.Text;

fOName := Edit3.Text;
i := length(fOName);
if ((i > 0) and (fOName[i] <> '\')) then fOName := fOName + '\';
fOName := fOName + Edit4.Text;

fNumber := 0;

fSize := Spin1.Value;
if      (Check1.Checked) then fSize := fSize * 1024
else if (Check2.Checked) then fSize := fSize * 1024 * 1024;

if (fSize < 1) then fSize := 1;

// use may cancel in the middle of it all

fCancel := false;
bCancelSplit.Enabled := true;

// divide by lines? chars? default is bytes

if      (Radio1.Checked) then SplitLines
else if (Radio2.Checked) then SplitChars
else                          SplitBytes;

// does no good to cancel now

bCancelSplit.Enabled := false;
end;



procedure TForm1.bCancelSplitClick(Sender: TObject);
begin
fCancel := true;
end;

{ ---------------------------------------------------------------------------- }
{ split a file by lines }

procedure TForm1.SplitLines;
var
    i,n:    integer;
    count:  integer;                    // number of lines copied
    iOK:    boolean;                    // input file ready?
    oOK:    boolean;                    // output file ready?
    itext:  TextFile;                   // input file
    otext:  TextFile;                   // output file
    ss:     string;                     // string to be copied
begin

// open input file

AssignFile(itext, fIName);
Reset(itext);
iOK := true;

// no output yet

oOK := false;
count := 0;
fNumber := 0;

// read and write until end of file

while ((not EOF(itext)) and (not fCancel)) do
    begin
    Application.ProcessMessages;

// make sure output is ready

    if (not oOK) then
        begin
        fNumber := fNumber + 1;
        ss := fOName + '.' + Base36(fNumber);
        AssignFile(otext, ss);
        Rewrite(otext);
        oOK := true;
        count  := 0;
        Static1.Caption := ss;
        end;

// read a line and write it out

    readln(itext, ss);
    writeln(otext, ss);
    count := count + 1;

// finished with that file?

    if (count >= fSize) then
        begin
        CloseFile(otext);
        oOK := false;
        end;
    end;    // while

// make sure last file is saved

if (oOK) then CloseFile(otext);

// done with input

CloseFile(itext);
end;

{ ---------------------------------------------------------------------------- }
{ split a file by characters }
{ char count included CR/LF at end of each line }

procedure TForm1.SplitChars;
var
    i,n:    integer;
    count:  integer;                    // number of chars copied
    iOK:    boolean;                    // input file ready?
    oOK:    boolean;                    // output file ready?
    itext:  TextFile;                   // input file
    otext:  TextFile;                   // output file
    ss:     string;                     // string to be copied
    fn:     string;                     // temp place for file name
begin

// open input file

AssignFile(itext, fIName);
Reset(itext);
iOK := true;

// no output yet

oOK := false;
count := 0;
fNumber := 0;

// read and write until end of file

while ((not EOF(itext)) and (not fCancel)) do
    begin
    Application.ProcessMessages;

// read a line and write it out -- count include CR/LF

    readln(itext, ss);
    count := count + length(ss) + 2;

// is it too big?

    if (count > fSize) then
        begin
        if (oOK) then CloseFile(otext);
        oOK := false;
        count := length(ss) + 2;
        end;

// there must be an output file

    if (not oOK) then
        begin
        fNumber := fNumber + 1;
        fn := fOName + '.' + Base36(fNumber);
        AssignFile(otext, fn);
        Rewrite(otext);
        oOK := true;
        Static1.Caption := fn;
        end;

// write it

    writeln(otext, ss);
    end;    // while


// make sure last file is saved

if (oOK) then CloseFile(otext);

// done with input

CloseFile(itext);
end;

{ ---------------------------------------------------------------------------- }
{ split a file by a fixed number of bytes }

procedure TForm1.SplitBytes;
var
    i,n:    integer;
    count:  integer;                    // number of chars copied
    need:   integer;                    // number of bytes needed
    actual: integer;                    // number of bytes actually read
    iOK:    boolean;                    // input file ready?
    oOK:    boolean;                    // output file ready?
    ifile:  File;                       // input file
    ofile:  File;                       // output file
    fn:     string;                     // temp place for file name
    buf:    TBuf128;                    // bytes to copy
begin

// open input file

AssignFile(ifile, fIName);
Reset(ifile, 1);
iOK := true;

// no output yet

oOK := false;
count := 0;
fNumber := 0;

// read and write until end of file

while ((not EOF(ifile)) and (not fCancel)) do
    begin
    Application.ProcessMessages;

// read a block of data up to 128 bytes

    need := fSize - count;
    if (need > 128) then need := 128;
    actual := 0;
    if (need > 0) then BlockRead(ifile, buf, need, actual);
    if (actual < 0) then actual := 0;

// is there an output file?

    if (not oOK) then
        begin
        fNumber := fNumber + 1;
        fn := fOName + '.' + Base36(fNumber);
        AssignFile(ofile, fn);
        Rewrite(ofile, 1);
        oOK := true;
        count := 0;
        Static1.Caption := fn;
        end;

// write out the buffer

    if (actual > 0) then BlockWrite(ofile, buf, actual);
    count := count + actual;

// did that fill up the output file?

    if (count >= fSize) then
        begin
        CloseFile(ofile);
        oOK := false;
        end;
    end;    // while


// make sure last file is saved

if (oOK) then CloseFile(ofile);

// done with input

CloseFile(ifile);
end;

{ ---------------------------------------------------------------------------- }
{ return a 3-character index base-36 }

function TForm1.Base36(aNumber: integer): string;
var
    i,d,n:  integer;
    s:      string;
begin
s := '';
n := aNumber;
if (n < 0) then n := 0;
for i := 1 to 3 do
    begin
    d := (n mod 36);
    n := n div 36;
    if ((0 <= d) and (d <= 9)) then s := chr(d + ord('0')) + s
    else                            s := chr(d - 10 + ord('A')) + s;
    end;

Base36 := s;
end;

{ ---------------------------------------------------------------------------- }

procedure TForm1.bMFileClick(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin
OpenDialog2.InitialDir := fHomeDir;
if (OpenDialog2.Execute) then
    begin
    s := OpenDialog2.Filename;
    Edit2.Text := s;
    end;

end;

procedure TForm1.Edit2Change(Sender: TObject);
var
    i,n:    integer;
    ok:     boolean;
    f:      File;
    s,t:    string;
begin
Static2.Caption := '';
Static3.Caption := '';
Edit5.Text      := '';
Edit6.Text      := '';

if (FileExists(Edit2.Text)) then
    begin

// default output directory and file name

    s := ExtractFilePath(Edit2.Text);
    i := length(s);
    if ((i > 0) and (s[i] <> '\')) then s := s + '\';
    Edit5.Text := s;

    s := ExtractFileName(Edit2.Text);
    i := pos('.', s);
    if (i > 0) then s := copy(s, 1, i-1);
    i := pos('$', s);
    if (i > 0) then s[i] := '.';
    Edit6.Text := s;

// find size of file

    AssignFile(f, Edit2.Text);
    Reset(f, 1);
    n := FileSize(f);
    CloseFile(f);

    Static2.Caption := IntToStr(n);

// how many files in the set?

    i := pos('.', Edit2.Text);
    if (i > 0) then s := copy(Edit2.Text, 1, i-1);

    i := 1;
    ok := true;
    while (ok) do
        begin
        i  := i + 1;
        t  := s + '.' + Base36(i);
        ok := FileExists(t);
        end;

    i := i - 1;
    Static3.Caption := IntToStr(i);
    end;

// something to merge

bMerge.Enabled := true;    
end;

{ ---------------------------------------------------------------------------- }

procedure TForm1.bMDirClick(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin
if (SelectDirectory('Destination Directory', '', s)) then
    begin
    i := length(s);
    if ((i > 0) and (s[i] <> '\')) then s := s + '\';
    Edit5.Text := s;
    end;
end;

{ ---------------------------------------------------------------------------- }
{ put the broken pieces back together }

procedure TForm1.bMergeClick(Sender: TObject);
var
    i,n:    integer;
    iOK:    boolean;                // input file ready
    oOK:    boolean;                // output file ready
    done:   boolean;                // more files to read?
    count:  integer;                // bytes read from input file
    ifile:  File;                   // input file
    ofile:  File;                   // output file
    fn:     string;                 // temp place for a file name
    buf:    TBuf128;                // bytes to copy
begin

// consolidate use data

fIName := Edit2.Text;
i := pos('.', fIName);
if (i > 0) then fIName := copy(fIName, 1, i-1);
Static4.Caption := '';

fOName := Edit5.Text;
i := length(fOName);
if ((i > 0) and (fOName[i] <> '\')) then fOName := fOName + '\';
fOName := fOName + Edit6.Text;
Static5.Caption := fOName;

fNumber := 0;
fSize := 0;

// allow user to cancel at some point

fCancel := false;
bCancelMerge.Enabled := true;

// open output file

AssignFile(ofile, fOName);
Rewrite(ofile, 1);
oOK := true;

// bu no input file yet

iOK := false;
count := 0;

// read until no more input files

done := false;
while ((not done) and (not fCancel)) do
    begin
    Application.ProcessMessages;

// is there an input file?

    if (not iOK) then
        begin
        fNumber := fNumber + 1;
        fn := fIName + '.' + Base36(fNumber);
        if (FileExists(fn)) then
            begin
            AssignFile(ifile, fn);
            Reset(ifile, 1);
            iOK := true;
            Static4.Caption := fn;
            end;
        end;

// still no input file? we must be finished

    if (not iOK) then done := true;

// read a buffer and write to output file

    n := 128;
    count := 0;
    if (iOK) then BlockRead(ifile, buf, n, count);

    if (count > 0) then BlockWrite(ofile, buf, count);

// was that the end of the input file?

    if ((count < n) and (iOK)) then
        begin
        CloseFile(ifile);
        iOK := false;
        end;
    end;    // while

// finished with input and output files

if (iOK) then CloseFile(ifile);
if (oOK) then CloseFile(ofile);

// no need to cancel now

bCancelMerge.Enabled := false;
end;



procedure TForm1.bCancelMergeClick(Sender: TObject);
begin
fCancel := true;
end;

end.
